@@ -114,17 +114,16 @@ void mosquitto_message_free(struct mosquitto_message **message)
 	_mosquitto_free(msg);
 }
 
-void _mosquitto_message_queue(struct mosquitto *mosq, struct mosquitto_message_all *message)
+void _mosquitto_message_queue(struct mosquitto *mosq, struct mosquitto_message_all *message, bool doinc)
 {
+	/* mosq->message_mutex should be locked before entering this function */
 	assert(mosq);
 	assert(message);
 
-	pthread_mutex_lock(&mosq->message_mutex);
 	mosq->queue_len++;
-	if(message->msg.qos > 0 && (mosq->max_inflight_messages == 0 || mosq->inflight_messages < mosq->max_inflight_messages)){
+	if(doinc == true && message->msg.qos > 0 && (mosq->max_inflight_messages == 0 || mosq->inflight_messages < mosq->max_inflight_messages)){
 		mosq->inflight_messages++;
 	}
-	pthread_mutex_unlock(&mosq->message_mutex);
 	message->next = NULL;
 	if(mosq->messages_last){
 		mosq->messages_last->next = message;
@@ -202,7 +201,9 @@ int _mosquitto_message_remove(struct mosquitto *mosq, uint16_t mid, enum mosquit
 			}
 			*message = cur;
 			mosq->queue_len--;
-			if(!mosq->messages){
+			if(cur->next == NULL){
+				mosq->messages_last = prev;
+			}else if(!mosq->messages){
 				mosq->messages_last = NULL;
 			}
 			if((cur->msg.qos == 2 && dir == mosq_md_in) || (cur->msg.qos > 0 && dir == mosq_md_out)){
@@ -236,7 +237,6 @@ int _mosquitto_message_remove(struct mosquitto *mosq, uint16_t mid, enum mosquit
 				pthread_mutex_unlock(&mosq->message_mutex);
 				return MOSQ_ERR_SUCCESS;
 			}
-			prev = cur;
 			cur = cur->next;
 		}
 		pthread_mutex_unlock(&mosq->message_mutex);
