@@ -555,9 +555,10 @@ int mosquitto_publish(struct mosquitto *mosq, int *mid, const char *topic, int p
 		message->msg.retain = retain;
 		message->dup = false;
 
-		_mosquitto_message_queue(mosq, message);
 		pthread_mutex_lock(&mosq->message_mutex);
+		_mosquitto_message_queue(mosq, message, false);
 		if(mosq->max_inflight_messages == 0 || mosq->inflight_messages < mosq->max_inflight_messages){
+			mosq->inflight_messages++;
 			if(qos == 1){
 				message->state = mosq_ms_wait_for_puback;
 			}else if(qos == 2){
@@ -882,9 +883,10 @@ int mosquitto_loop_forever(struct mosquitto *mosq, int timeout, int max_packets)
 		}else{
 			pthread_mutex_unlock(&mosq->state_mutex);
 
-			reconnect_delay = mosq->reconnect_delay;
-			if(reconnect_delay > 0 && mosq->reconnect_exponential_backoff){
-				reconnect_delay *= mosq->reconnect_delay*reconnects*reconnects;
+			if(mosq->reconnect_delay > 0 && mosq->reconnect_exponential_backoff){
+				reconnect_delay = mosq->reconnect_delay*reconnects*reconnects;
+			}else{
+				reconnect_delay = mosq->reconnect_delay;
 			}
 
 			if(reconnect_delay > mosq->reconnect_delay_max){
