@@ -158,7 +158,9 @@ int _mosquitto_handle_publish(struct mosquitto *mosq)
 		case 2:
 			rc = _mosquitto_send_pubrec(mosq, message->msg.mid);
 			message->state = mosq_ms_wait_for_pubrel;
-			_mosquitto_message_queue(mosq, message);
+			pthread_mutex_lock(&mosq->message_mutex);
+			_mosquitto_message_queue(mosq, message, true);
+			pthread_mutex_unlock(&mosq->message_mutex);
 			return rc;
 		default:
 			return MOSQ_ERR_PROTOCOL;
