@@ -86,6 +86,7 @@ static struct mosquitto *_db_find_or_add_context(struct mosquitto_db *db, const
 			}
 		}
 		context->id = _mosquitto_strdup(client_id);
+		context->db_index = i;
 	}
 	if(last_mid){
 		context->last_mid = last_mid;
@@ -486,6 +487,7 @@ static int _db_client_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 	int rc = 0;
 	struct mosquitto *context;
 	time_t disconnect_t;
+	struct _clientid_index_hash *new_cih;
 
 	read_e(db_fptr, &i16temp, sizeof(uint16_t));
 	slen = ntohs(i16temp);
@@ -518,6 +520,16 @@ static int _db_client_chunk_restore(struct mosquitto_db *db, FILE *db_fptr)
 
 	_mosquitto_free(client_id);
 
+	if(!rc){
+		new_cih = _mosquitto_malloc(sizeof(struct _clientid_index_hash));
+		if(!new_cih){
+			_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: Out of memory.");
+			return MOSQ_ERR_NOMEM;
+		}
+		new_cih->id = context->id;
+		new_cih->db_context_index = context->db_index;
+		HASH_ADD_KEYPTR(hh, db->clientid_index_hash, context->id, strlen(context->id), new_cih);
+	}
 	return rc;
 error:
 	_mosquitto_log_printf(NULL, MOSQ_LOG_ERR, "Error: %s.", strerror(errno));
